/*
 * Copyright (c) 2012-2014 Wind River Systems, Inc.
 *
 * SPDX-License-Identifier: Apache-2.0
 */

#include <zephyr.h>
#include <sys/printk.h>
#include <device.h>
#include <drivers/pwm.h>
#include <drivers/gpio.h>
#include <drivers/pwm.h>
// #include <drivers/adc.h>

// 
#define DEBOUNCE_CYCLES 5

// Parameters for PWM control
static const struct pwm_dt_spec pwm_front = PWM_DT_SPEC_GET(DT_ALIAS(mcfront));
static const struct pwm_dt_spec pwm_rear = PWM_DT_SPEC_GET(DT_ALIAS(mcrear));
#define PWM_PERIOD PWM_MSEC(1U)   // set PWMs to 10kHz

// Variables for power status
static int powerLevel;
static int debounceCounter;


// Define the pins in the Nordic connection
// Bar Light
#define LED0_NODE DT_ALIAS(ledbar0)
#define LED1_NODE DT_ALIAS(ledbar1)
#define LED2_NODE DT_ALIAS(ledbar2)
#define LED3_NODE DT_ALIAS(ledbar3)
// Button lights
#define LEDBL_NODE DT_ALIAS(ledbuttonl)
#define LEDBR_NODE DT_ALIAS(ledbuttonr)
// Generate structs for LEDs
static const struct gpio_dt_spec led0 = GPIO_DT_SPEC_GET(LED0_NODE, gpios);
static const struct gpio_dt_spec led1 = GPIO_DT_SPEC_GET(LED1_NODE, gpios);
static const struct gpio_dt_spec led2 = GPIO_DT_SPEC_GET(LED2_NODE, gpios);
static const struct gpio_dt_spec led3 = GPIO_DT_SPEC_GET(LED3_NODE, gpios);
static const struct gpio_dt_spec ledBL = GPIO_DT_SPEC_GET(LEDBL_NODE, gpios);
static const struct gpio_dt_spec ledBR = GPIO_DT_SPEC_GET(LEDBR_NODE, gpios);


// // switches
#define SW0_NODE	DT_ALIAS(buttonl)
#define SW1_NODE	DT_ALIAS(buttonr)
static const struct gpio_dt_spec button0 = GPIO_DT_SPEC_GET(SW0_NODE, gpios);
static const struct gpio_dt_spec button1 = GPIO_DT_SPEC_GET(SW1_NODE, gpios);

// Define an interupt for the button being pressed
void button1_pressed(const struct device *dev, struct gpio_callback *cb, uint32_t pins)
{
	if (debounceCounter < 1){	
		gpio_pin_set_dt(&ledBR, 1);
		powerLevel++;
		if(powerLevel > 4) {
			powerLevel = 4;
		} 
		debounceCounter = DEBOUNCE_CYCLES;
	}
}

void button0_pressed(const struct device *dev, struct gpio_callback *cb, uint32_t pins)
{
    if (debounceCounter < 1){	
		gpio_pin_set_dt(&ledBL, 1);
		powerLevel--;
		if(powerLevel < 0) {
			powerLevel = 0;
		}
		debounceCounter = DEBOUNCE_CYCLES;
	}
}

void setLedBar(){
	if(powerLevel > 0) {
		gpio_pin_set_dt(&led0, 1);
	} else {
		gpio_pin_set_dt(&led0, 0);
	}
	if(powerLevel > 1) {
		gpio_pin_set_dt(&led1, 1);
	} else {
		gpio_pin_set_dt(&led1, 0);
	}
	if(powerLevel > 2) {
		gpio_pin_set_dt(&led2, 1);
	} else {
		gpio_pin_set_dt(&led2, 0);
	}
	if(powerLevel > 3) {
		gpio_pin_set_dt(&led3, 1);
	} else {
		gpio_pin_set_dt(&led3, 0);
	}
}

// Define a variable of type static struct gpio_callback for buttons 1 and 2
static struct gpio_callback button0_cb_data;
static struct gpio_callback button1_cb_data;

void main(void)
{

	int ret;
	powerLevel = 0;
	debounceCounter = 2;
	
	// Set up PWMs
	if (!device_is_ready(pwm_front.dev) || !device_is_ready(pwm_rear.dev)) {
		return;
	}
	ret = pwm_set_dt(&pwm_front, PWM_PERIOD, PWM_PERIOD / 2U);
	if(ret > 0) {
		return;
	}
	ret = pwm_set_dt(&pwm_rear, PWM_PERIOD, PWM_PERIOD / 2U);
	if(ret > 0) {
		return;
	}
	

	// Initialize the LEDs and confirm they are ready to use
	if (!device_is_ready(led0.port)) {
		return;
	}
	
	ret = gpio_pin_configure_dt(&led0, GPIO_OUTPUT_ACTIVE);
	ret += gpio_pin_configure_dt(&led1, GPIO_OUTPUT_ACTIVE);
	ret += gpio_pin_configure_dt(&led2, GPIO_OUTPUT_ACTIVE);
	ret += gpio_pin_configure_dt(&led3, GPIO_OUTPUT_ACTIVE);
	ret += gpio_pin_configure_dt(&ledBL, GPIO_OUTPUT_ACTIVE);
	ret += gpio_pin_configure_dt(&ledBR, GPIO_OUTPUT_ACTIVE);
	if (ret < 0) {
		return;
	}

	// Verify that the device is ready for use 
	if (!device_is_ready(button0.port)) {
		return;
	}
	if (!device_is_ready(button1.port)) {
		return;
	}
	ret = gpio_pin_configure_dt(&button0, GPIO_INPUT);
	ret = gpio_pin_configure_dt(&button1, GPIO_INPUT);
	if (ret < 0) {
		return;
	}


	setLedBar();

	// Configure the interrupt for button 0 
	ret = gpio_pin_interrupt_configure_dt(&button0, GPIO_INT_EDGE_FALLING );
	gpio_init_callback(&button0_cb_data, button0_pressed, BIT(button0.pin)); 
	gpio_add_callback(button0.port, &button0_cb_data);

	// Configure the interrupt for button 1 
	ret = gpio_pin_interrupt_configure_dt(&button1, GPIO_INT_EDGE_FALLING );
	gpio_init_callback(&button1_cb_data, button1_pressed, BIT(button1.pin)); 
	gpio_add_callback(button1.port, &button1_cb_data);



	// Main loop
	while(1){

        k_msleep(100); // Put the main thread to sleep for 100ms for power optimization
		// Reset LEDs
		setLedBar();
		// Lower the debound counter
		--debounceCounter;
		if(debounceCounter < 0){
			debounceCounter = 0;
			gpio_pin_set_dt(&ledBL, 0);
			gpio_pin_set_dt(&ledBR, 0);
		};

	}
}
